import time
import functools
import numpy as np

def time_compute(fun):
    @functools.wraps(fun)
    def wrapper(*args, **kwargs):
        start_time = time.time()
        tmp = fun(*args, **kwargs)
        end_time = time.time()
        print('{} cost {} s.'.format(fun.__name__, end_time-start_time))
        return tmp

    return wrapper

def time_avg_compute(fun):
    @functools.wraps(fun)
    def wrapper(*args, **kwargs):
        times = []
        tmp = None
        for _ in range(20):
            start_time = time.time()
            tmp = fun(*args, **kwargs)
            end_time = time.time()
            times.append(end_time-start_time)

        times = np.array(times)
        times = times[5:]
        print('{} avg cost {} s.'.format(fun.__name__, times.mean()))
        return tmp

    return wrapper


import cv2
import os

img_dir = '/Users/yongkangfan/Downloads'
count = 0
for img_name in os.listdir(img_dir):
    if img_name.split('.')[1] != 'png':
        continue
    img_path = '{}/{}'.format(img_dir, img_name)
    print(img_path)
    img = cv2.imread(img_path)
    img = cv2.resize(img, (640*2, 640))
    cv2.imshow('img res', img)

    output_name = 'output_{}.png'.format(count)
    cv2.imwrite('{}/{}'.format(img_dir, output_name), img)
    count += 1
    cv2.waitKey(0)


import pandas as pd
import tensorflow as tf

TRAIN_URL = "http://download.tensorflow.org/data/iris_training.csv"
TEST_URL = "http://download.tensorflow.org/data/iris_test.csv"

CSV_COLUMN_NAMES = ['SepalLength', 'SepalWidth',
                    'PetalLength', 'PetalWidth', 'Species']
SPECIES = ['Setosa', 'Versicolor', 'Virginica']

def maybe_download():
    train_path = tf.keras.utils.get_file(TRAIN_URL.split('/')[-1], TRAIN_URL)
    test_path = tf.keras.utils.get_file(TEST_URL.split('/')[-1], TEST_URL)

    return train_path, test_path

def load_data(y_name='Species'):
    """Returns the iris dataset as (train_x, train_y), (test_x, test_y)."""
    train_path, test_path = maybe_download()

    train = pd.read_csv(train_path, names=CSV_COLUMN_NAMES, header=0)
    train_x, train_y = train, train.pop(y_name)

    test = pd.read_csv(test_path, names=CSV_COLUMN_NAMES, header=0)
    test_x, test_y = test, test.pop(y_name)

    return (train_x, train_y), (test_x, test_y)


def train_input_fn(features, labels, batch_size):
    """An input function for training"""
    # Convert the inputs to a Dataset.
    dataset = tf.data.Dataset.from_tensor_slices((dict(features), labels))

    # Shuffle, repeat, and batch the examples.
    dataset = dataset.shuffle(1000).repeat(100).batch(batch_size)

    # Return the dataset.
    return dataset


def eval_input_fn(features, labels, batch_size):
    """An input function for evaluation or prediction"""
    features=dict(features)
    if labels is None:
        # No labels, use only features.
        inputs = features
    else:
        inputs = (features, labels)

    # Convert the inputs to a Dataset.
    dataset = tf.data.Dataset.from_tensor_slices(inputs)

    # Batch the examples
    assert batch_size is not None, "batch_size must not be None"
    dataset = dataset.batch(batch_size)

    # Return the dataset.
    return dataset


# The remainder of this file contains a simple example of a csv parser,
#     implemented using the `Dataset` class.

# `tf.parse_csv` sets the types of the outputs to match the examples given in
#     the `record_defaults` argument.
CSV_TYPES = [[0.0], [0.0], [0.0], [0.0], [0]]

def _parse_line(line):
    # Decode the line into its fields
    fields = tf.decode_csv(line, record_defaults=CSV_TYPES)

    # Pack the result into a dictionary
    features = dict(zip(CSV_COLUMN_NAMES, fields))

    # Separate the label from the features
    label = features.pop('Species')

    return features, label


def csv_input_fn(csv_path, batch_size):
    # Create a dataset containing the text lines.
    dataset = tf.data.TextLineDataset(csv_path).skip(1)

    # Parse each line.
    dataset = dataset.map(_parse_line)

    # Shuffle, repeat, and batch the examples.
    dataset = dataset.shuffle(1000).repeat().batch(batch_size)

    # Return the dataset.
    return dataset


import matplotlib.pyplot as plt
import tensorflow as tf

image_raw_data = tf.gfile.FastGFile('./Kyrie_Irving.jpg', 'rb',).read()

with tf.Session() as sess:
    img_data = tf.image.decode_jpeg(image_raw_data)

    img_convert = tf.image.convert_image_dtype(img_data, dtype=tf.float32)
    # it seems the random convert methon need [0,255] range date instead of [0, 1.0] range.
    img_convert = tf.image.random_brightness(img_convert, max_delta=0.5)

    img_rotate = tf.contrib.image.rotate(tf.expand_dims(img_convert/255.0, 0), tf.constant([0.5]))*255.0

    batched = tf.expand_dims(img_convert, 0)
    box = tf.constant([[[0.07, 0.45, 0.3, 0.6]]])
    img_box = tf.image.draw_bounding_boxes(images=batched, boxes=box)

    # the bbox_for_draw is another expression of [begin, begin+size],
    # not the bounding box in the new image.
    begin, size, bbox_for_draw = tf.image.sample_distorted_bounding_box(
        tf.shape(img_convert), bounding_boxes=box, min_object_covered=0.1
    )
    distorted_image = tf.slice(img_convert, begin=begin, size=size)
    comparing_image = tf.image.draw_bounding_boxes(images=tf.expand_dims(img_convert, 0), boxes=bbox_for_draw)

    img_data, img_convert, distorted_image, comparing_image = \
        sess.run([img_data, img_rotate, distorted_image, comparing_image])

    img_list = [img_data, img_convert[0], distorted_image, comparing_image[0]]

    for i in range(len(img_list)):
        plt.subplot(2, 2, i+1)
        plt.imshow(img_list[i])
        plt.title('img {}'.format(i))
    plt.show()

    # img_data = tf.image.convert_image_dtype(resizd, dtype=tf.uint8)
    # encode_image = tf.image.encode_jpeg(img_data)
    # with tf.gfile.GFile('./ouwen', 'wb') as f:
    #     f.write(encode_image.eval())


import os
import shutil

def clear_dir(dir_path):
    if isinstance(dir_path, list):
        for dir_p in dir_path:
            if dir_p is None:
                continue
            clear_dir(dir_p)

    if os.path.exists(dir_path):
        shutil.rmtree(dir_path)
    os.mkdir(dir_path)


# --------------------------------------------------------
# Fast R-CNN
# Copyright (c) 2015 Microsoft
# Licensed under The MIT License [see LICENSE for details]
# Written by Ross Girshick
# --------------------------------------------------------

import numpy as np


def nms(dets, scores, iou_threshold, num_boxes):
    y1 = dets[:, 0]
    x1 = dets[:, 1]
    y2 = dets[:, 2]
    x2 = dets[:, 3]

    areas = (x2 - x1 + 1) * (y2 - y1 + 1)
    order = scores.argsort()[::-1]

    keep = []
    while order.size > 0:
        i = order[0]
        keep.append(i)
        if len(keep) >= num_boxes:
            break
        xx1 = np.maximum(x1[i], x1[order[1:]])
        yy1 = np.maximum(y1[i], y1[order[1:]])
        xx2 = np.minimum(x2[i], x2[order[1:]])
        yy2 = np.minimum(y2[i], y2[order[1:]])

        w = np.maximum(0.0, xx2 - xx1 + 1)
        h = np.maximum(0.0, yy2 - yy1 + 1)
        inter = w * h
        ovr = inter / (areas[i] + areas[order[1:]] - inter)

        inds = np.where(ovr <= iou_threshold)[0]
        order = order[inds + 1]

    return keep


import cv2
import numpy as np
import tensorflow as tf
import hashlib
import os
from dataset.DataSetRead import DataSetRead
from dataset.dataset_util import IoU


# coarse convert
def convert_ellipse_2_rectangle(data_lines):
    gt_bboxes = []
    for data_line in data_lines:
        print(data_line)
        major_axis_radius, minor_axis_radius, angle, center_x, center_y, _ = map(float, data_line.strip().split())
        gt_bboxes.append([center_x-minor_axis_radius, center_y-major_axis_radius, 2*minor_axis_radius, 2*major_axis_radius])
    return np.array(gt_bboxes, dtype=np.int)

class FDDB(DataSetRead):
    def __init__(self):
        super(FDDB, self).__init__()
        self.dataset_dir = '/home/xuhai/disk2/ykfan/data/FDDB'


    def _read_annotations(self, data_type, limit_count=-1):
        assert isinstance(data_type, int), 'data type should be int type.'
        annotation_list = list()
        annotation_txt = '{}/FDDB-folds/FDDB-fold-{:0>2d}-ellipseList.txt'.format(self.dataset_dir, data_type)
        annotations = open(annotation_txt).readlines()
        index = 0
        while index < len(annotations):
            if 0 < limit_count < len(annotation_list):
                break

            annotation_name = annotations[index].strip()
            bbox_len = int(annotations[index + 1])

            annotation_val = dict()
            annotation_val['file_name'] = annotation_name
            annotation_val['id'] = annotation_name
            annotation_val['dataset'] = 'fddb'
            annotation_val['data'] = convert_ellipse_2_rectangle(annotations[index + 2:index + 2 + bbox_len])
            annotation_val['img_dir'] = self.dataset_dir
            annotation_list.append(annotation_val)
            index += bbox_len + 2

        return annotation_list


    def decode_annotation_val(self, annotation_val):
        full_path = '{}/{}.jpg'.format(annotation_val['img_dir'], annotation_val['file_name'])
        image = cv2.imread(full_path)
        assert image is not None, 'no image {} exists!'.format(full_path)
        image_height, image_width, _ = image.shape
        with tf.gfile.GFile(full_path, 'rb') as fid:
            encoded_jpg = fid.read()
        key = hashlib.sha256(encoded_jpg).hexdigest()

        xmin = []
        xmax = []
        ymin = []
        ymax = []
        is_crowd = []
        category_names = []
        category_ids = []
        area = []
        num_annotations_skipped = 0
        for object_annotation in annotation_val['data']:
            x, y, width, height = object_annotation
            # (x, y, width, height) = tuple(object_annotations['bbox'])
            if width <= 0 or height <= 0:
                tf.logging.debug('skip for width {}, height {}'.format(width, height))
                num_annotations_skipped += 1
                continue
            if x + width > image_width or y + height > image_height:
                tf.logging.debug('skip for beyond image range.')
                num_annotations_skipped += 1
                continue
            xmin.append(float(x) / image_width)
            xmax.append(float(x + width) / image_width)
            ymin.append(float(y) / image_height)
            ymax.append(float(y + height) / image_height)
            category_id = 0
            category_names.append(self.category_index[category_id]['name'].encode('utf8'))
            cv2.rectangle(image, (x, y), (x + width, y + height), self.category_index[category_id]['color'])
            # unused label
            category_ids.append(category_id)
            is_crowd.append(0)
            area.append(0.0)

        annotation_val['image_height'] = image_height
        annotation_val['image_width'] = image_width
        annotation_val['encoded_jpg'] = encoded_jpg
        annotation_val['key'] = key
        annotation_val['xmin'] = xmin
        annotation_val['xmax'] = xmax
        annotation_val['ymin'] = ymin
        annotation_val['ymax'] = ymax
        annotation_val['is_crowd'] = is_crowd
        annotation_val['category_names'] = category_names
        annotation_val['category_ids'] = category_ids
        annotation_val['area'] = area
        annotation_val['num_annotations_skipped'] = num_annotations_skipped
        annotation_val['verify_img'] = image
        return annotation_val


    def evaluate(self, detect_estimator, iou_threshold=0.3, verify_dir=None, limit_count=-1):
        annotation_list = self.read_annotations(range(1, 11), limit_count=limit_count)
        total_precision_sum = 0
        total_precision_tp = 0
        total_recall_sum = 0
        total_recall_tp = 0

        for index, annotation in enumerate(annotation_list):
            precision_tp = 0
            image_path = '{}/{}.jpg'.format(annotation['img_dir'], annotation['file_name'])
            img = cv2.imread(image_path)
            assert img is not None, '{} should not be None'.format(annotation['file_name'])
            pred_bboxes = detect_estimator(image_path)
            gt_bboxes = annotation['data']
            gt_mask = np.zeros([len(gt_bboxes)])

            for pred_bbox in pred_bboxes:
                iou_vals = IoU(pred_bbox, gt_bboxes)
                pred_bbox = [int(_) for _ in pred_bbox]
                # print iou_vals
                max_index = np.argmax(iou_vals)
                if iou_vals[max_index] >= iou_threshold:
                    gt_mask[max_index] = 1
                    precision_tp += 1
                    cv2.rectangle(img, (pred_bbox[0], pred_bbox[1]), (pred_bbox[2], pred_bbox[3]), (0, 255, 0))
                else:
                    cv2.rectangle(img, (pred_bbox[0], pred_bbox[1]), (pred_bbox[2], pred_bbox[3]), (0, 0, 255))

            for gt_bbox in gt_bboxes:
                cv2.rectangle(img, (gt_bbox[0], gt_bbox[1]), (gt_bbox[2], gt_bbox[3]), (255, 0, 0))

            recall_tp = np.sum(gt_mask)

            total_precision_sum += len(pred_bboxes)
            total_precision_tp += precision_tp
            total_recall_sum += len(gt_bboxes)
            total_recall_tp += recall_tp

            if index % 100 == 0 or index == len(annotation_list) - 1:
                if not os.path.exists(verify_dir):
                    os.mkdir(verify_dir)
                cv2.imwrite('{}/verify_{}.jpg'.format(verify_dir, index), img)
                print('image precision {:.1f} %, recall {:.1f} %'.format(
                    float(precision_tp) * 100.0 / (float(len(pred_bboxes)) + 0.000001),
                    float(recall_tp) * 100.0 / (float(len(gt_bboxes)) + 0.000001)
                ))
                print( '{}/{} total precision {:.1f} %, recall {:.1f} %'.format(
                    index,
                    len(annotation_list),
                    float(total_precision_tp) * 100.0 / float(total_precision_sum),
                    float(total_recall_tp) * 100.0 / float(total_recall_sum)
                ))


if __name__ == '__main__':
    pass
    # def detector_estimator(img):
    #     return {'bboxes': np.array([[0, 0, 20, 20], [0, 0, 30, 30],])}
    #
    # evaluate(detect_estimator=detector_estimator, limit_count=20)



import contextlib2
import tensorflow as tf
from tqdm import tqdm
import cv2
import dataset.dataset_util as dataset_util
from common_utils.dir_utils import clear_dir

class DataSetRead(object):
    def __init__(self):
        print('data set read init.')
        self.category_index = [{'name':'face', 'color':(255, 0, 0)}, {'name':'head', 'color':(0, 255, 0)}, {'name':'shoulder', 'color':(0, 0, 255)},]

    def _read_annotations(self, data_type, limit_count=-1):
        raise NotImplementedError('virtual function')

    def read_annotations(self, data_type_list, limit_count=-1):
        assert type(data_type_list) == list, 'data type list should be type list'
        total_list = list()
        for data_type in data_type_list:
            annotation_list = self._read_annotations(data_type, limit_count)
            total_list.extend(annotation_list)
        return total_list

    def decode_annotation_val(self, annotation_val):
        raise NotImplementedError('virtual function')

    def create_tf_example(self, annotation_val, verify_dict=None, feature_spec_dict=None, idx=0):
        annotation_val = self.decode_annotation_val(annotation_val)
        if verify_dict is not None and idx%verify_dict['verify_count']==0:
            cv2.imwrite('{}/verify_{}.jpg'.format(verify_dict['verify_dir'], idx), annotation_val['verify_img'])

        feature_dict = {
            'image/height':
                dataset_util.int64_feature(annotation_val['image_height']),
            'image/width':
                dataset_util.int64_feature(annotation_val['image_width']),
            'image/filename':
                dataset_util.bytes_feature(annotation_val['file_name'].encode('utf8')),
            'image/source_id':
                dataset_util.bytes_feature(str(annotation_val['id']).encode('utf8')),
            'image/key/sha256':
                dataset_util.bytes_feature(annotation_val['key'].encode('utf8')),
            'image/encoded':
                dataset_util.bytes_feature(annotation_val['encoded_jpg']),
            'image/format':
                dataset_util.bytes_feature('jpeg'.encode('utf8')),
            'image/object/bbox/xmin':
                dataset_util.float_list_feature(annotation_val['xmin']),
            'image/object/bbox/xmax':
                dataset_util.float_list_feature(annotation_val['xmax']),
            'image/object/bbox/ymin':
                dataset_util.float_list_feature(annotation_val['ymin']),
            'image/object/bbox/ymax':
                dataset_util.float_list_feature(annotation_val['ymax']),
            'image/object/class/text':
                dataset_util.bytes_list_feature(annotation_val['category_names']),
            'image/object/is_crowd':
                dataset_util.int64_list_feature(annotation_val['is_crowd']),
            'image/object/area':
                dataset_util.float_list_feature(annotation_val['area']),
        }

        if feature_spec_dict:
            spec_feature_dict = dict()
            for spec_key, spec_val in feature_spec_dict.items():
                spec_feature_dict[spec_key] = feature_dict[spec_val]
            feature_dict = spec_feature_dict

        example = tf.train.Example(features=tf.train.Features(feature=feature_dict))
        return annotation_val['key'], example, annotation_val['num_annotations_skipped']

    def create_tf_records(self, data_type_list, output_path, num_shards, verify_dict=None, feature_spec_dict=None):
        '''
        :param data_type_list:
        :param output_path:
        :param num_shards:
        :param verify_dict: {'verify_count': 1000, 'verify_dir': 'data/verify_train'},
        :param feature_spec_dict:
        :return:
        '''

        clear_dir(output_path)
        if verify_dict is not None:
            clear_dir(verify_dict['verify_dir'])

        annotations_list = self.read_annotations(data_type_list, limit_count=100)
        with contextlib2.ExitStack() as tf_record_close_stack:
            output_tfrecords = dataset_util.open_sharded_output_tfrecords(
                tf_record_close_stack, output_path, num_shards)

            total_num_annotations_skipped = 0
            for idx, annotation_val in enumerate(tqdm(annotations_list)):
                _, tf_example, num_annotations_skipped = self.create_tf_example(annotation_val, verify_dict, feature_spec_dict, idx)
                assert tf_example is not None, 'tf example should not be None!'
                total_num_annotations_skipped += num_annotations_skipped
                shard_idx = idx % num_shards
                output_tfrecords[shard_idx].write(tf_example.SerializeToString())
            tf.logging.info('Finished writing, skipped %d annotations.', total_num_annotations_skipped)

    def evaluate(self, detect_estimator, iou_threshold=0.3, verify_dir=None, limit_count=-1):
        raise NotImplementedError('virtual function')




import numpy as np
import os
import tensorflow as tf

def IoU(box, boxes, iou_type='join'):
    """Compute IoU between detect box and gt boxes

    Parameters:
    ----------
    box: numpy array , shape (5, ): x1, y1, x2, y2, score
        input box
    boxes: numpy array, shape (n, 4): x1, y1, x2, y2
        input ground truth boxes

    Returns:
    -------
    ovr: numpy.array, shape (n, )
        IoU
    """
    box_area = (box[2] - box[0] + 1) * (box[3] - box[1] + 1)
    area = (boxes[:, 2] - boxes[:, 0] + 1) * (boxes[:, 3] - boxes[:, 1] + 1)
    xx1 = np.maximum(box[0], boxes[:, 0])
    yy1 = np.maximum(box[1], boxes[:, 1])
    xx2 = np.minimum(box[2], boxes[:, 2])
    yy2 = np.minimum(box[3], boxes[:, 3])

    # compute the width and height of the bounding box
    w = np.maximum(0, xx2 - xx1 + 1)
    h = np.maximum(0, yy2 - yy1 + 1)

    inter = w * h
    if iou_type == 'inter':
        return inter / area
    ovr = inter / (box_area + area - inter)
    return ovr


def convert_to_square(bbox):
    """Convert bbox to square

    Parameters:
    ----------
    bbox: numpy array , shape n x 5
        input bbox

    Returns:
    -------
    square bbox
    """
    square_bbox = bbox.copy()

    h = bbox[:, 3] - bbox[:, 1] + 1
    w = bbox[:, 2] - bbox[:, 0] + 1
    max_side = np.maximum(h,w)
    square_bbox[:, 0] = bbox[:, 0] + w*0.5 - max_side*0.5
    square_bbox[:, 1] = bbox[:, 1] + h*0.5 - max_side*0.5
    square_bbox[:, 2] = square_bbox[:, 0] + max_side - 1
    square_bbox[:, 3] = square_bbox[:, 1] + max_side - 1
    return square_bbox


def get_sub_path_image_name(root_dir, index):
    sub_dir = '{}/{:0>4d}'.format(root_dir, index//10000)
    print('{}'.format(sub_dir))
    if not os.path.exists(sub_dir):
        print('mkdir {}'.format(sub_dir))
        os.mkdir(sub_dir)
    return '{}/{:0>4d}.jpg'.format(sub_dir, index)


def int64_feature(value):
  return tf.train.Feature(int64_list=tf.train.Int64List(value=[value]))


def int64_list_feature(value):
  return tf.train.Feature(int64_list=tf.train.Int64List(value=value))


def bytes_feature(value):
  return tf.train.Feature(bytes_list=tf.train.BytesList(value=[value]))


def bytes_list_feature(value):
  return tf.train.Feature(bytes_list=tf.train.BytesList(value=value))


def float_list_feature(value):
  return tf.train.Feature(float_list=tf.train.FloatList(value=value))


def open_sharded_output_tfrecords(exit_stack, base_path, num_shards):
  """Opens all TFRecord shards for writing and adds them to an exit stack.

  Args:
    exit_stack: A context2.ExitStack used to automatically closed the TFRecords
      opened in this function.
    base_path: The base path for all shards
    num_shards: The number of shards

  Returns:
    The list of opened TFRecords. Position k in the list corresponds to shard k.
  """
  tf_record_output_filenames = [
      '{}/shard-{:04d}.tfrecords'.format(base_path, idx)
      for idx in range(num_shards)
  ]

  tfrecords = [
      exit_stack.enter_context(tf.python_io.TFRecordWriter(file_name))
      for file_name in tf_record_output_filenames
  ]

  return tfrecords


import cv2
import tensorflow as tf
import hashlib
import os
from dataset.DataSetRead import DataSetRead

class WiderFace(DataSetRead):
    def __init__(self):
        super(WiderFace, self).__init__()
        self.dataset_dir = '/home/xuhai/disk2/ykfan/data/WiderFace'

    def _read_annotations(self, data_type, limit_count=-1):
        assert data_type in ['train', 'val', ], 'annotatin type for wider face should be train or val.'
        annotation_txt = '{}/wider_face_split/wider_face_{}_bbx_gt.txt'.format(self.dataset_dir, data_type)
        img_dir = '{}/WIDER_{}/images'.format(self.dataset_dir, data_type)
        annotation_list = list()

        annotations = open(annotation_txt).readlines()
        index = 0
        while index < len(annotations):
            annotation_name = annotations[index].strip()
            bbox_len = int(annotations[index + 1])

            annotation_val = dict()
            annotation_val['file_name'] = annotation_name
            annotation_val['id'] = annotation_name
            annotation_val['dataset'] = 'WiderFace'
            annotation_val['data'] = annotations[index + 2:index + 2 + bbox_len]
            annotation_val['img_dir'] = img_dir
            annotation_list.append(annotation_val)
            index += bbox_len + 2

            if 0 < limit_count < len(annotation_list):
                break
        tf.logging.info('wider face {} dataset, with size {}'.format(data_type, len(annotation_list)))
        return annotation_list


    def decode_annotation_val(self, annotation_val):
        full_path = os.path.join(annotation_val['img_dir'], annotation_val['file_name'])
        image = cv2.imread(full_path)
        assert image is not None, 'no image {} exists!'.format(full_path)
        image_height, image_width, _ = image.shape
        with tf.gfile.GFile(full_path, 'rb') as fid:
            encoded_jpg = fid.read()
        key = hashlib.sha256(encoded_jpg).hexdigest()

        xmin = []
        xmax = []
        ymin = []
        ymax = []
        is_crowd = []
        category_names = []
        category_ids = []
        area = []
        num_annotations_skipped = 0
        for object_annotations in annotation_val['data']:
            x, y, width, height = list(map(int, object_annotations.split()))[:4]
            # (x, y, width, height) = tuple(object_annotations['bbox'])
            if width <= 0 or height <= 0:
                tf.logging.debug('skip for width {}, height {}'.format(width, height))
                num_annotations_skipped += 1
                continue
            if x + width > image_width or y + height > image_height:
                tf.logging.debug('skip for beyond image range.')
                num_annotations_skipped += 1
                continue
            xmin.append(float(x) / image_width)
            xmax.append(float(x + width) / image_width)
            ymin.append(float(y) / image_height)
            ymax.append(float(y + height) / image_height)
            category_id = 0
            category_names.append(self.category_index[category_id]['name'].encode('utf8'))
            cv2.rectangle(image, (x, y), (x + width, y + height), self.category_index[category_id]['color'])
            # unused label
            category_ids.append(category_id)
            is_crowd.append(0)
            area.append(0.0)

        annotation_val['image_height'] = image_height
        annotation_val['image_width'] = image_width
        annotation_val['encoded_jpg'] = encoded_jpg
        annotation_val['key'] = key
        annotation_val['xmin'] = xmin
        annotation_val['xmax'] = xmax
        annotation_val['ymin'] = ymin
        annotation_val['ymax'] = ymax
        annotation_val['is_crowd'] = is_crowd
        annotation_val['category_names'] = category_names
        annotation_val['category_ids'] = category_ids
        annotation_val['area'] = area
        annotation_val['num_annotations_skipped'] = num_annotations_skipped
        annotation_val['verify_img'] = image
        return annotation_val


if __name__ == '__main__':
    pass
    # train_txt = '{}/wider_face_split/wider_face_train_bbx_gt.txt'.format(dataset_dir)
    #
    # train_annotations = open(train_txt).readlines()
    #
    # index = 0
    # while index < len(train_annotations):
    #     img_path = train_annotations[index].strip()
    #     img_path = '{}/WIDER_train/images/{}'.format(dataset_dir, img_path)
    #     print(img_path)
    #     img = cv2.imread(img_path)
    #     pose_atypical_num = 0
    #
    #     bbox_len = int(train_annotations[index + 1])
    #     for bbox_index in range(bbox_len):
    #         x1, y1, w, h, _, _, __, _, _, pose = map(int, train_annotations[index + 2 + bbox_index].split())
    #         if pose == 1 and w > 20 and h > 20:
    #             pose_atypical_num += 1
    #             cv2.rectangle(img, (x1, y1), (x1 + w, y1 + h), (0, 255, 0))
    #
    #     if pose_atypical_num > 0:
    #         cv2.imshow('wider face', img)
    #         cv2.waitKey(0)
    #
    #     index += bbox_len + 2





# class TypedField:
#     def __init__(self, _type):
#         self._type = _type
#
#     def __get__(self, instance, cls):
#         if instance is None:
#             return self
#         else:
#             return getattr(instance, self.name)
#
#     def __set_name__(self, cls, name):
#         self.name = name
#
#     def __set__(self, instance, value):
#         print '========'
#         if not isinstance(value, self._type):
#             raise TypeError('Expected' + str(self._type))
#         instance.__dict__[self.name] = value
#
# class Person:
#     age = TypedField(int)
#     name = TypedField(str)
#
#     def __init__(self, age, name):
#         self.age = age
#         self.name = name
#
# jack = Person(15, 'Jack')
# print jack.name
# jack.age = Person(13, 'lucy')
# print jack.age

# # test int float convert
# # 0.666666666667
# # 0
# # 0.666666666667
#
# print 2.0/3
# print 2/3
# print 2/3.0


# import numpy as np
# print np.random.beta(1.5, 1.5)
#
# label1 = np.array([[1.0, 2.0, 1.0, 2.9], [2.3, 3.2, 2.2, 4.4]])
#
# y1 = np.hstack((label1, np.full((label1.shape[0], 1), np.random.beta(1.5, 1.5))))
# print y1

# from tqdm import tqdm
# import time
# for i in tqdm(range(1000)):
#     print('hello')
#     time.sleep(1)

# from __future__ import print_function
# print('love', 'panda', sep=', ')

# from __future__ import print_function
# try:
#     raise Exception('test')
# except Exception as e_val:
#     print('catch: {}'.format(e_val))
#
# print(2//3)

import os

for root, dirs, files in os.walk('/home/ykfan/code/head_pose/my_utils/test', topdown=False):
    print('==============')
    print(root)
    print(dirs)
    print(files)



# import os
# os.environ['CUDA_VISIBLE_DEVICES'] = ''
# import tensorflow as tf

# import tensorflow as tf
# # test reuce_mean
# xx = tf.constant([[[1., 1, 1, 1],
#                    [2., 2, 2, 2],
#                    [3., 3, 3, 3]],
#
#                   [[4, 4, 4, 4],
#                    [5, 5, 5, 5],
#                    [6, 6, 6, 6]]])
# m3 = tf.reduce_mean(xx, axis=(1, 2), keepdims=True) # [2.5 2.5 2.5]
# m4 = tf.multiply(xx, m3)
#
# b_val = tf.constant([True, False, False, True])
# f_val = tf.to_float(b_val)
#
# # with tf.control_dependencies([tf.assert_equal(f_val, tf.constant([1.0, 0.0, 0.0, 1.0]))]):
# #     i_val = tf.to_int32(f_val)
# with tf.control_dependencies([tf.Print(f_val, [f_val], message='print f_val ')]):
#     i_val = tf.to_int32(f_val)
#
# add_val = i_val + 2
# # seem tensorflow assert dont need to run.
# with tf.Session() as sess:
#     sess.run(tf.global_variables_initializer())
#     print(xx.get_shape())
#     print(m3.get_shape())
#     print(sess.run(m3))
#     print(sess.run(m4))
#     # print(sess.run(add_val))
#     # for op in tf.get_default_graph().get_operations():
#     #     print op



# def tf_add_one(tf_val):
#     b = tf.constant([1.0], name='add_one')
#     return tf_val + b
#
# with tf.variable_scope('s1'):
#     input_val = tf.constant([2.0], name='input')
#     a = tf.get_variable('plus_val', [1])
#     val = input_val*a
#     with tf.variable_scope('s2', reuse=True):
#         val = tf_add_one(val)
#         c = tf.get_variable('plus_val', [1], )
#         val = val*c
#
# for op in tf.get_default_graph().get_operations():
#     print op


# # gpu test
# import tensorflow as tf
#
# c = []
# with tf.device('/cpu:0'):
#     # device name not int the op name or scope.
#     a = tf.get_variable("a", [2, 2], initializer=tf.random_uniform_initializer(-1, 1))
#     b = tf.get_variable("b", [2, 2], initializer=tf.random_uniform_initializer(-1, 1))
#
# with tf.device('/gpu:0'):
#     c.append(tf.matmul(a, b))
#
# with tf.device('/gpu:1'):
#     c.append(a + b)
#
# with tf.device('/cpu:0'):
#     sum = tf.add_n(c)
#
# sess = tf.Session(config=tf.ConfigProto(log_device_placement=True, allow_soft_placement=True))
#
# init = tf.global_variables_initializer()
# sess.run(init)
#
# print(sess.run(sum))
#
# #operation and variable different.
# for op in tf.get_default_graph().get_operations():
#     print op
# for var in tf.get_collection(tf.GraphKeys.VARIABLES):
#     print var


# # name scope and variable scope
# import tensorflow as tf
#
# with tf.variable_scope('a'):
#     with tf.name_scope('b'):
#         with tf.variable_scope('c'):
#             # variable_scope work on get_variable
#             val = tf.get_variable('var', shape=[1], initializer=tf.zeros_initializer)
#             # variable_scop and name_scope work on tf.Variable
#             n = tf.Variable([2.0], name='n')
#             # variable_scop and name_scope work on op tensor.
#             d = val+n
# print val
# print n
# print d


# # test the wing loss
# import tensorflow as tf
# import math
# tf.enable_eager_execution()
#
# w = 10.0
# epsilon = 2.0
# label = tf.constant([1, 0, -2, -2])
# landmark_pred = tf.constant([
#     [0.49, 0.26, 0.45, 0.52],
#     [0.51, 0.24, 0.46, 0.52],
#     [0.36, 0.43, 0.47, 0.51],
#     [0.46, 0.22, 0.53, 0.55],
# ])
#
# lanmark_target = tf.constant([
#     [0.5, 0.25, 0.5, 0.5],
#     [0.5, 0.25, 0.5, 0.5],
#     [0.5, 0.25, 0.5, 0.5],
#     [0.5, 0.25, 0.5, 1.0],
# ])
#
# with tf.name_scope('wing_loss'):
#     ones = tf.ones_like(label, dtype=tf.float32)
#     zeros = tf.zeros_like(label, dtype=tf.float32)
#     valid_inds = tf.where(tf.equal(label, -2), ones, zeros)
#
#     c = w * (1.0 - math.log(1.0 + w / epsilon))
#     absolute_x = tf.abs(landmark_pred - lanmark_target)
#     losses = tf.where(
#         tf.greater(w, absolute_x),
#         w * tf.log(1.0 + absolute_x / epsilon),
#         absolute_x - c
#     )
#     losses = tf.reduce_sum(losses, axis=1)
#     num_valid = tf.reduce_sum(valid_inds)
#     keep_num = tf.cast(num_valid, dtype=tf.int32)
#     losses = losses * valid_inds
#     _, k_index = tf.nn.top_k(losses, k=keep_num)
#     losses = tf.gather(losses, k_index)
#     loss = tf.reduce_mean(losses)


# # test tf.gradients
# import tensorflow as tf
# with tf.name_scope('scope_1'):
#     a = tf.Variable(1.0, name='a')
# with tf.name_scope('scope_2'):
#     b = tf.Variable(1.0, name='b')
# c = a + b
# d = a + b
#
# loss = c + d
# gradients = tf.gradients(loss, [a, b])
# gradients_with_stop = tf.gradients(loss, [a, b], stop_gradients=[d])
# init = tf.global_variables_initializer()
#
# opt = tf.train.GradientDescentOptimizer(0.1).minimize(loss)
# # print tf.get_collection(tf.GraphKeys.TRAINABLE_VARIABLES)
#
# selected_var_list = tf.get_collection(tf.GraphKeys.TRAINABLE_VARIABLES, scope='scope_1')
# selected_opt = tf.train.GradientDescentOptimizer(0.1).minimize(loss, var_list=selected_var_list)
#
# with tf.Session() as sess:
#     sess.run(init)
#     print sess.run([gradients])
#     print sess.run([gradients_with_stop])
#     print sess.run([opt, a, b])
#     sess.run(init)
#     print sess.run([selected_opt, a, b])


# #test operstion and tensor
# import tensorflow as tf
# with tf.name_scope('n1'):
#     a = tf.Variable(1.0, name='a')
#     b = tf.Variable(1.0, name='b')
#     c = tf.Variable(1.0, name='c')
#
#     d = a+b
#     e = tf.add(b, c, name='e')
#
#     f = d+e
#
# for op in tf.get_default_graph().get_operations():
#     print op.name
# # tensor.name = op.name + ':0'
# print e

# import tensorflow as tf
#
# a = tf.get_variable('a', shape =[], initializer=tf.zeros_initializer, dtype=tf.int32)
# b = tf.get_variable('b', shape =[], initializer=tf.zeros_initializer, dtype=tf.int32)
#
# init = tf.global_variables_initializer()
#
# with tf.Session() as sess:
#     sess.run(init)
#     for i in range(5):
#         debug_val = tf.assign(b, a)
#         with tf.control_dependencies([debug_val]):
#             a = b+1
#         print sess.run([a, b])


# import tensorflow as tf
#
# h = tf.Variable(48, dtype=tf.int32)
# x = tf.constant(128, dtype=tf.int32)
# new_h = x * tf.to_int32(tf.ceil(tf.to_float(h)/tf.to_float(x)))
#
# # h = tf.Variable(48, dtype=tf.int32)
# # x = 128
# # new_h = x * tf.to_int32(tf.ceil(h / x))
#
# init = tf.global_variables_initializer()
# with tf.Session() as sess:
#     sess.run(init)
#     print(sess.run(new_h))
#
# import tensorflow as tf
# import copy
# from tensorflow.core.framework import graph_pb2
#
# def get_add_graph():
#     with tf.Graph().as_default() as add_graph:
#         a = tf.placeholder(tf.float32, (), name='a')
#         b = tf.Variable([2.0], trainable=False, name='b')
#         c = tf.add(a, b, name='c')
#
#         for op in add_graph.get_operations():
#             print(op.name)
#
#         for var in tf.get_collection(tf.GraphKeys.GLOBAL_VARIABLES):
#             print(var)
#
#         init = tf.global_variables_initializer()
#         with tf.Session(graph=add_graph) as sess:
#             sess.run(init)
#             print('\n', sess.run([c], feed_dict={'a:0': 1.0}), '\n')
#     return add_graph
#
# def get_plus_grpah():
#     with tf.Graph().as_default() as plus_graph:
#         d = tf.placeholder(tf.float32, (), name='d')
#         e = tf.Variable([2.0], trainable=False, name='e')
#         f = tf.multiply(d, e, name='f')
#     return plus_graph
#
# def combineGraph(add_graph, plus_graph):
#
#     out = graph_pb2.GraphDef()
#     for node in add_graph.node:
#         out.node.extend([copy.deepcopy(node)])
#
#     for node in plus_graph.node:
#         if node.name == 'd':
#             continue
#         idx = 0
#         for x in node.input:
#              if x == 'd':
#                  #idx = node.input.index(x)
#                  node.input.remove(x)
#                  node.input.insert(idx, 'c')
#                  break
#              idx = idx+1
#         out.node.extend([copy.deepcopy(node)])
#
#     out.library.CopyFrom(add_graph.library)
#     out.versions.CopyFrom(add_graph.versions)
#     return out
#
# # add_graph = get_add_graph()
# # add_graph_def = add_graph.as_graph_def().node
# # for node in add_graph_def:
# #     print node
#
# combine_graph = combineGraph(get_add_graph().as_graph_def(), get_plus_grpah().as_graph_def())
# with tf.Graph().as_default() as run_graph:
#     tf.import_graph_def(combine_graph)
#     for op in run_graph.get_operations():
#         print(op.name)
#
#     var_b = run_graph.get_tensor_by_name('import/b:0')
#     var_d = run_graph.get_tensor_by_name('import/e:0')
#
#     tf.add_to_collection(tf.GraphKeys.GLOBAL_VARIABLES, var_b)
#     tf.add_to_collection(tf.GraphKeys.GLOBAL_VARIABLES, var_d)
#
#     for var in tf.get_collection(tf.GraphKeys.GLOBAL_VARIABLES):
#         print(var)
#
#
#     res_tensor_c = run_graph.get_tensor_by_name('import/c:0')
#     res_tensor_f = run_graph.get_tensor_by_name('import/f:0')
#
#
#     init = tf.global_variables_initializer()
#     with tf.Session(graph=run_graph) as sess:
#         sess.run(iniy = tf.matmul([[37.0, -23.0], [1.0, 4.0]], tf.random_uniform([2, 2]))
#
# with tf.Session() as sess:
#   # Define options for the `sess.run()` call.
#   options = tf.RunOptions()
#   options.output_partition_graphs = True
#   options.trace_level = tf.RunOptions.FULL_TRACE
#
#   # Define a container for the returned metadata.
#   metadata = tf.RunMetadata()
#
#   sess.run(y, options=options, run_metadata=metadata)
#
#   # Print the subgraphs that executed on each device.
#   print(metadata.partition_graphs)
#
#   # Print the timings of each operation that executed.
#   print(metadata.step_stats)t)
#         print(sess.run(res_tensor_c, feed_dict={'import/a:0': 1.0}))


# import tensorflow as tf
#
# a = tf.constant([1.0, 2.0])
# print_op = tf.print(a)
# with tf.control_dependencies([print_op]):
#     b = a + 1
#
# with tf.Session() as sess:
#     print(sess.run([b]))


import tensorflow as tf
y = tf.matmul([[37.0, -23.0], [1.0, 4.0]], tf.random_uniform([2, 2]))

with tf.Session() as sess:
  # Define options for the `sess.run()` call.
  options = tf.RunOptions()
  options.output_partition_graphs = True
  options.trace_level = tf.RunOptions.FULL_TRACE

  # Define a container for the returned metadata.
  metadata = tf.RunMetadata()

  sess.run(y, options=options, run_metadata=metadata)

  # Print the subgraphs that executed on each device.
  print(metadata.partition_graphs)

  # Print the timings of each operation that executed.
  print(metadata.step_stats)


# estimator test

from __future__ import print_function
import shutil
import os
import tensorflow as tf
import test.iris_data as iris_data

batch_size = 100
num_step = 2000

model_dir = './log'
# clear the model dir
if os.path.exists(model_dir):
    shutil.rmtree(model_dir)
os.mkdir(model_dir)

def model_fun(features, labels, mode, params):
    net = tf.feature_column.input_layer(features, params['feature_cloumns'])
    for units in params['hidden_layers']:
        net = tf.layers.dense(net, units=units, activation=tf.nn.relu)
    logits = tf.layers.dense(net, units=params['num_classes'])
    predicted_classes = tf.argmax(logits, 1)
    if mode == tf.estimator.ModeKeys.PREDICT:
        predictions = {
            'class_ids': predicted_classes[:, tf.newaxis],
            'probabilities': tf.nn.softmax(logits),
            'logits': logits
        }
        return tf.estimator.EstimatorSpec(mode, predictions=predictions)

    accuracy = tf.metrics.accuracy(labels, predicted_classes)
    loss = tf.losses.sparse_softmax_cross_entropy(labels, logits)
    tf.summary.scalar('accuracy', accuracy[0])

    if mode == tf.estimator.ModeKeys.EVAL:
        metrics = {'accuracy': accuracy}
        return tf.estimator.EstimatorSpec(mode, loss=loss, eval_metric_ops=metrics)

    assert mode == tf.estimator.ModeKeys.TRAIN
    global_step = tf.train.get_global_step()
    boundaries = [1000, 1500]
    values = [0.03, 0.01, 0.005]
    learning_rate = tf.train.piecewise_constant(global_step, boundaries, values)
    optimizer = tf.train.AdamOptimizer(learning_rate=learning_rate)
    train_op = optimizer.minimize(loss, global_step=global_step)
    return tf.estimator.EstimatorSpec(mode, train_op=train_op, loss=loss)



def main(unuse_argv):
    (train_x, train_y), (test_x, test_y) = iris_data.load_data()
    MY_FEATURE_COLUMN = [tf.feature_column.numeric_column(_) for _ in train_x.keys()]

    # params are passed to model_fun.
    config = tf.estimator.RunConfig(model_dir=model_dir)
    iris_estimator = tf.estimator.Estimator(model_fn=model_fun,
                                            config=config,
                                            params={'feature_cloumns': MY_FEATURE_COLUMN,
                                                    'hidden_layers': [10, 10],
                                                    'num_classes': 3})
    # when train input dataset repeat(n), n is not empty, the train_and_evaluate will eval every epoch train input dataset run out.
    train_spec = tf.estimator.TrainSpec(input_fn=lambda :iris_data.train_input_fn(train_x, train_y, batch_size), max_steps=num_step)
    eval_spec = tf.estimator.EvalSpec(input_fn=lambda : iris_data.eval_input_fn(test_x, test_y, batch_size))
    tf.estimator.train_and_evaluate(iris_estimator, train_spec, eval_spec)

    # #train_and_evaluate finally return the eval_spec result.
    # eval_result = tf.estimator.train_and_evaluate(iris_estimator, train_spec, eval_spec)
    # print(eval_result)

    # Generate predictions from the model
    expected = ['Setosa', 'Versicolor', 'Virginica']
    predict_x = {
        'SepalLength': [5.1, 5.9, 6.9],
        'SepalWidth': [3.3, 3.0, 3.1],
        'PetalLength': [1.7, 4.2, 5.4],
        'PetalWidth': [0.5, 1.5, 2.1],
    }

    predictions = iris_estimator.predict(lambda : iris_data.eval_input_fn(predict_x, None, batch_size))
    for pred_dict, expe in zip(predictions, expected):
        tf.logging.debug(pred_dict)
        pred_class = int(pred_dict['class_ids'])
        pred_prob = pred_dict['probabilities'][pred_class]
        tf.logging.info('pred class {} with probability {:.3f}, with expect {}'.format(iris_data.SPECIES[pred_class], pred_prob, expe))

if __name__ == '__main__':
    tf.logging.set_verbosity(tf.logging.INFO)
    tf.app.run(main)


import logging
import cv2
import tensorflow as tf
import hashlib
import json
import os
import dataset.dataset_util as dataset_util

# {"ID": "284193,faa9000f2678b5e", "gtboxes":
#     [{"tag": "person", "hbox": [123, 129, 63, 64], "head_attr": {"ignore": 0, "occ": 1, "unsure": 0}, "fbox": [61, 123, 191, 453], "vbox": [62, 126, 154, 446], "extra": {"box_id": 0, "occ": 1}},
#      {"tag": "person", "hbox": [214, 97, 58, 74], "head_attr": {"ignore": 0, "occ": 1, "unsure": 0}, "fbox": [165, 95, 187, 494], "vbox": [175, 95, 140, 487], "extra": {"box_id": 1, "occ": 1}},
#      {"tag": "person", "hbox": [318, 109, 58, 68], "head_attr": {"ignore": 0, "occ": 1, "unsure": 0}, "fbox": [236, 104, 195, 493], "vbox": [260, 106, 170, 487], "extra": {"box_id": 2, "occ": 1}},
#      {"tag": "person", "hbox": [486, 119, 61, 74], "head_attr": {"ignore": 0, "occ": 0, "unsure": 0}, "fbox": [452, 110, 169, 508], "vbox": [455, 113, 141, 501], "extra": {"box_id": 3, "occ": 1}},
#      {"tag": "person", "hbox": [559, 105, 53, 57], "head_attr": {"ignore": 0, "occ": 0, "unsure": 0}, "fbox": [520, 95, 163, 381], "vbox": [553, 98, 70, 118], "extra": {"box_id": 4, "occ": 1}},
#      {"tag": "person", "hbox": [596, 40, 72, 83], "head_attr": {"ignore": 0, "occ": 0, "unsure": 0}, "fbox": [546, 39, 202, 594], "vbox": [556, 39, 171, 588], "extra": {"box_id": 5, "occ": 1}},
#      {"tag": "person", "hbox": [731, 139, 69, 83], "head_attr": {"ignore": 0, "occ": 0, "unsure": 0}, "fbox": [661, 132, 183, 510], "vbox": [661, 132, 183, 510], "extra": {"box_id": 6, "occ": 0}}]}

dataset_dir = '/home/ykfan/data/CrowdHuman'
category_index = [{'name':'face', 'color':(255, 0, 0)}, {'name':'head', 'color':(0, 255, 0)}, {'name':'shoulder', 'color':(0, 0, 255)},]


def box_pading(box, vbox):
    if box[0] < vbox[0]:
        box[0] = vbox[0]
    if box[1] < vbox[1]:
        box[1] = vbox[1]
    if box[2] > vbox[0]+vbox[2]:
        box[2] = vbox[0]+vbox[2]
    if box[3] > vbox[1]+vbox[3]:
        box[3] = vbox[1]+vbox[3]
    return list(map(int, box))


# return format is [is_use, shoulder xmin, ymin, xmax, ymax, face xmin ymin, xmax, ymax]
def box_shoulder(hbox, vbox):
    logging.debug('hbox {}'.format(hbox))
    logging.debug('vbox {}'.format(vbox))
    hbox = list(map(float, hbox))
    vbox = list(map(float, vbox))
    hbox_w = hbox[2]
    hbox_h = hbox[3]

    shoulder_box = [hbox[0]-0.5*hbox_w, hbox[1], hbox[0]+1.5*hbox_w, hbox[1]+2.0*hbox_h]
    return box_pading(shoulder_box, vbox)

def get_shoulder_boxes(annotation_val, image_width, image_height):
    gt_shouder_boxes = []
    for object_annotations in annotation_val['data']:
        # print gtboxes[index]['tag']
        if not object_annotations['tag'] == 'person':
            continue
        head_box = list(map(int, object_annotations['hbox']))
        x, y, width, height = head_box
        if width <= 0 or height <= 0:
            continue
        if x + width > image_width or y + height > image_height:
            continue

        virt_box = list(map(int, object_annotations['vbox']))
        shoulder_box = box_shoulder(head_box, virt_box)
        gt_shouder_boxes.append(shoulder_box)
    return gt_shouder_boxes

def read_annotations(annotation_type, limit_count = -1):
    if isinstance(annotation_type, list):
        annotations = list()
        for type_val in annotation_type:
            annotations.extend(read_annotations(type_val, limit_count))
        return annotations

    assert annotation_type in ['train', 'val'], 'annotation type should be in type train,val.'
    annotation_txt = 'annotation_{}.odgt'.format(annotation_type)
    annotation_lines = open(os.path.join(dataset_dir, annotation_txt)).readlines()
    img_dir = '{}/Images'.format(dataset_dir)

    annotation_list = []
    for annotation in annotation_lines:
        # print annotation
        annotation = json.loads(annotation)
        annotation_val = dict()

        annotation_val['id'] = annotation['ID']
        annotation_val['file_name'] = '{}.jpg'.format(annotation_val['id'])
        annotation_val['dataset'] = 'CrowdHuman'
        annotation_val['data'] = annotation['gtboxes']
        annotation_val['img_dir'] = img_dir
        annotation_list.append(annotation_val)

        if 0 < limit_count < len(annotation_list):
            break
    tf.logging.info('crowd human {} dataset, with size {}'.format(annotation_type, len(annotation_list)))
    return annotation_list

def decode_annotation(annotation_val):
    full_path = os.path.join(annotation_val['img_dir'], annotation_val['file_name'])
    image = cv2.imread(full_path)
    assert image is not None, 'no image {} exists!'.format(full_path)
    image_height, image_width, _ = image.shape

    gt_head_boxes = []
    gt_shoulder_boxes = []
    for object_annotations in annotation_val['data']:
        # print gtboxes[index]['tag']
        if not object_annotations['tag'] == 'person':
            continue
        head_box = list(map(int, object_annotations['hbox']))
        x, y, width, height = head_box
        if width <= 0 or height <= 0:
            tf.logging.debug('skip for width {}, height {}'.format(width, height))
            continue
        if x + width > image_width or y + height > image_height:
            tf.logging.debug('skip for beyond image range.')
            continue
        virt_box = list(map(int, object_annotations['vbox']))
        shoulder_box = box_shoulder(head_box, virt_box)
        # now gt boxes are all [x1, y1, x2, y2] format
        gt_head_boxes.append([x, y, x+width, y+height])
        gt_shoulder_boxes.append(shoulder_box)

    annotation_val['gt_head_boxes'] = gt_head_boxes
    annotation_val['gt_shoulder_boxes'] = gt_shoulder_boxes
    annotation_val['image'] = image
    return annotation_val


def create_tf_example(annotation_val, verify_dict=None):
    filename = annotation_val['file_name']
    image_id = annotation_val['id']

    full_path = os.path.join(annotation_val['img_dir'], filename)
    image = cv2.imread(full_path)
    assert image is not None, 'no image {} exists!'.format(full_path)
    image_height, image_width, _ = image.shape
    with tf.gfile.GFile(full_path, 'rb') as fid:
        encoded_jpg = fid.read()
    key = hashlib.sha256(encoded_jpg).hexdigest()

    xmin = []
    xmax = []
    ymin = []
    ymax = []
    is_crowd = []
    category_names = []
    category_ids = []
    area = []
    num_annotations_skipped = 0
    for object_annotations in annotation_val['data']:
        # print gtboxes[index]['tag']
        if not object_annotations['tag'] == 'person':
            continue

        head_box = list(map(int, object_annotations['hbox']))
        x, y, width, height = head_box
        if width <= 0 or height <= 0:
            tf.logging.debug('skip for width {}, height {}'.format(width, height))
            num_annotations_skipped += 1
            continue
        if x + width > image_width or y + height > image_height:
            tf.logging.debug('skip for beyond image range.')
            num_annotations_skipped += 1
            continue

        xmin.append(float(x) / image_width)
        xmax.append(float(x + width) / image_width)
        ymin.append(float(y) / image_height)
        ymax.append(float(y + height) / image_height)
        category_id = 1
        cv2.rectangle(image, (x,y), (x+width, y+height), category_index[category_id]['color'])
        category_names.append(category_index[category_id]['name'].encode('utf8'))
        #unused label
        is_crowd.append(0)
        area.append(0.0)

        virt_box = list(map(int, object_annotations['vbox']))
        shoulder_box = box_shoulder(head_box, virt_box)
        x1, y1, x2, y2 = shoulder_box

        xmin.append(float(x1) / image_width)
        xmax.append(float(x2) / image_width)
        ymin.append(float(y1) / image_height)
        ymax.append(float(y2) / image_height)
        category_id = 2
        cv2.rectangle(image, (x1,y1), (x2, y2), category_index[category_id]['color'])
        category_names.append(category_index[category_id]['name'].encode('utf8'))
        #unused label
        is_crowd.append(0)
        area.append(0.0)

    if verify_dict is not None:
        cv2.imwrite(verify_dict['save_path'], image)

    feature_dict = {
        'image/height':
            dataset_util.int64_feature(image_height),
        'image/width':
            dataset_util.int64_feature(image_width),
        'image/filename':
            dataset_util.bytes_feature(filename.encode('utf8')),
        'image/source_id':
            dataset_util.bytes_feature(str(image_id).encode('utf8')),
        'image/key/sha256':
            dataset_util.bytes_feature(key.encode('utf8')),
        'image/encoded':
            dataset_util.bytes_feature(encoded_jpg),
        'image/format':
            dataset_util.bytes_feature('jpeg'.encode('utf8')),
        'image/object/bbox/xmin':
            dataset_util.float_list_feature(xmin),
        'image/object/bbox/xmax':
            dataset_util.float_list_feature(xmax),
        'image/object/bbox/ymin':
            dataset_util.float_list_feature(ymin),
        'image/object/bbox/ymax':
            dataset_util.float_list_feature(ymax),
        'image/object/class/text':
            dataset_util.bytes_list_feature(category_names),
        'image/object/is_crowd':
            dataset_util.int64_list_feature(is_crowd),
        'image/object/area':
            dataset_util.float_list_feature(area),
    }

    example = tf.train.Example(features=tf.train.Features(feature=feature_dict))
    return key, example, num_annotations_skipped

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    verify_dir = 'verify'
    count = 0
    os.mkdir(verify_dir)
    for val in read_annotations(['train', 'val']):
        count += 1
        if count % 1000 != 0:
            continue

        val = decode_annotation(val)
        img = val['image']
        for box in val['gt_head_boxes']:
            cv2.rectangle(img, (box[0], box[1]), (box[2], box[3]), (0, 255, 0))
        for box in val['gt_shoulder_boxes']:
            cv2.rectangle(img, (box[0], box[1]), (box[2], box[3]), (0, 0, 255))

        cv2.imwrite('{}/out_{}.jpg'.format(verify_dir, count), img)
        logging.info('verify image {}'.format(count))





import numpy as np
import tensorflow as tf
import matplotlib.pyplot as plt

def distort_color(image, color_ordering=0):
    if color_ordering == 0:
        image = tf.image.random_brightness(image, max_delta=15. / 255.)
        image = tf.image.random_saturation(image, lower=0.9, upper=1.1)
        image = tf.image.random_hue(image, max_delta=0.1)
        image = tf.image.random_contrast(image, lower=0.9, upper=1.1)
    elif color_ordering == 1:
        image = tf.image.random_contrast(image, lower=0.5, upper=1.7)
        image = tf.image.random_hue(image, max_delta=0.2)
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
    elif color_ordering == 2:
        image = tf.image.random_hue(image, max_delta=0.2)
        image = tf.image.random_saturation(image, lower=0.5, upper=1.5)
        image = tf.image.random_contrast(image, lower=0.5, upper=1.7)
        image = tf.image.random_brightness(image, max_delta=32. / 255.)
    return image

def preprocess_for_train(image, height, width, bbox):
    if bbox is None:
        bbox = tf.constant([0.0, 0.0, 1.0, 1.0], dtype=tf.float32, shape=[1, 1, 4])

    if image.dtype != tf.float32:
        image = tf.image.convert_image_dtype(image, dtype=tf.float32)

    bbox_begin, bbox_size, _ = tf.image.sample_distorted_bounding_box(tf.shape(image), bounding_boxes=bbox)
    distort_image = tf.slice(image, bbox_begin, bbox_size)
    distort_image = tf.image.resize_images(distort_image, [height, width], method=np.random.randint(4))
    distort_image = tf.image.random_flip_left_right(distort_image)
    distort_image = distort_color(distort_image, np.random.randint(3))

    return distort_image

if __name__ == '__main__':
    image_raw_data = tf.gfile.FastGFile('F:/PycharmProjects/tmp/data/a.jpg', 'rb').read()

    with tf.Session() as sess:
        img_data = tf.image.decode_jpeg(image_raw_data)
        boxes = tf.constant([[[0.05, 0.05, 0.9, 0.7], [0.35, 0.47, 0.5, 0.56]]])

        for i in range(6):
            result = preprocess_for_train(img_data, 299, 299, boxes)
            plt.imshow(result.eval())
            plt.show()


import cv2
import logging

image_wh = (1280, 720)

def record(record_name):
    cap = cv2.VideoCapture(0)
    assert cap.isOpened(), 'Please open the mac camera!'
    out = cv2.VideoWriter(record_name, cv2.VideoWriter_fourcc(*'XVID'), 10, image_wh)

    while True:
        ret, frame = cap.read()
        frame = cv2.resize(frame, image_wh)
        out.write(frame)
        cv2.imshow('camera', frame)
        key = cv2.waitKey(30)
        if key & 0xFF == ord(' '):
            break
    cap.release()
    out.release()
    cv2.destroyAllWindows()

def show(show_name):
    cap = cv2.VideoCapture(show_name)
    assert cap.isOpened(), 'Please chebck {} exists!'.format(video_name)
    while True:
        ret, frame = cap.read()
        if frame is None:
            logging.info('read video finished.')
            break
        cv2.imshow('camera', frame)
        key = cv2.waitKey(30)
        if key & 0xFF == ord('b'):
            break
        if key & 0xFF == ord(' '):
            cv2.waitKey(0)
    cap.release()
    cv2.destroyAllWindows()

if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    video_name = 'pose_angle.avi'
    # record(video_name)
    show(video_name)

    # img = cv2.imread('out_3044.jpg')
    # cv2.imshow('img', img)
    # cv2.waitKey(0)
    # print cv2.__version__

